<?php
namespace FiloBlu\Flow\Model\Connector;
/**
 * Created by PhpStorm.
 * User: marcovalenziano
 * Date: 07/05/15
 * Time: 12:40
 */

class Ftp extends AbstractConnector{

    protected $host;
    protected $username;
    protected $password;
    protected $passive = 0;
    public $type = 'FTP';
    public $ftpDir;
    public $incomingDir;
    public $localDir;
    protected $_logger;


    /*
     * Init FTP Class retrieving configuration
     */
    public function __construct(
        \FiloBlu\Flow\Helper\LoggerProvider $_loggerProvider
    ){
        parent::__construct($_loggerProvider);
        $this->_logger = $_loggerProvider->getLogger();
    }

    /**
     * @param $config
     */
    public function setConfig($config)
    {
        $this->_config = $config;

        $this->host = (String) @$this->_config['host'];
        $this->username = (String) @$this->_config['username'];
        $this->password = (String) @$this->_config['password'];
        $this->passive= (String)@$this->_config['passive'];
        $this->ftpDir= (String) @$this->_config['ftp_dir'];
        if(isset($this->_config['incomingDir'])) {
            $this->incomingDir = $this->_config['incomingDir'];
        }
        $this->localDir= BP . "/var/" .(String) $this->_config['local_dir'];
        $this->connection=null;

        $this->_mask = (String) $this->_config['regex'];

        if(!file_exists($this->localDir)){
            mkdir($this->localDir);
        }

    }

    /**
     * @return mixed|void
     */
    public function fetchResources()
    {

    }

    /**
     * @return mixed
     */
    public function getLocalDir() {
        return $this->localDir;
    }

    /**
     * @return resource
     * @throws \Exception
     */
    public function openConnection() {

        $host = $this->host;

        if (!$this->connection) {
            if(!$host) {
                $this->_logger->info(__METHOD__.' -> Missing HOST');
                throw new \Exception('Missing HOST');
            }


            $this->connection = ftp_connect($host);
            if (!$this->connection) {
                $this->_logger->info(__METHOD__.' -> FTP connection failed');
                throw new \Exception('FTP connection failed');
            }
            $this->login($this->username,$this->password);
        }
        return $this->connection;
    }

    /**
     * @return mixed|void
     */
    public function getConnection() {

        $this->openConnection();

        return $this->connection;

    }

    /**
     * @param null $user
     * @param null $pass
     * @throws \Exception
     */
    public function login($user=null, $pass=null) {

        $login_result = ftp_login($this->connection, $user, $pass);
        if (!$login_result) {
            $this->_logger->info(__METHOD__.' -> FTP login failed');
            throw new \Exception('FTP login failed');
        }

        //Modalità passiva attivata
        ftp_pasv($this->connection, $this->passive);
    }

    /**
     *
     * @param type $connection Connection
     * @param string $dir Directory to list
     * @return type List of file names
     */
    public function listDir($dir){

        $this->_logger->info(__METHOD__);

        $connection = $this->connection;

        ftp_pasv($connection, true);

        putenv('TMPDIR='.getenv('TMPDIR'));

        if (ftp_chdir($connection,$dir)) {
            $list = ftp_nlist($connection, ".");
        }
        else {
            $list = array();
        }

        return $list;
    }

    /**
     * Scarica un file dall'FTP
     * @param type $connection
     * @param string $local_file
     * @param string $server_file
     * @throws \Exception
     */
    public function downloadFile($local_file, $server_file, $mode = FTP_ASCII) {

        $this->_logger->info(__METHOD__);

        $connection = $this->connection;

        if(!ftp_chdir($connection,$this->ftpDir)) {
            $this->_logger->info(__METHOD__ . "FTP HELPER: function ftp_chdir failed - downloading {$server_file} > {$local_file}");
            throw new \Exception("FTP HELPER: function ftp_chdir failed - downloading {$server_file} > {$local_file}");
        }

        $res = ftp_get($connection, $local_file, $server_file, $mode);
        if(!$res) {
            $this->_logger->info(__METHOD__ . "FTP HELPER: function ftp_get failed - downloading {$server_file} > {$local_file}");
            throw new \Exception("FTP HELPER: function ftp_get failed - downloading {$server_file} > {$local_file}");
        }
        else {
            chmod ( $local_file , 0777 );
            return true;
        }
    }

    /**
     * @param $remote_file
     * @param $xmlPath
     * @param int $mode
     * @return bool
     * @throws \Exception
     */
    public function uploadFile($remote_file, $xmlPath,$mode = FTP_ASCII) {

        $connection = $this->connection;

        $this->_logger->info("Upload del file {$remote_file}");

        $res = ftp_put($connection, $remote_file, $xmlPath, $mode); //FTP_ASCII per i file di testo in generale

        if(!$res) {
            $this->_logger->info("FTP HELPER: upload failed {$remote_file} > {$xmlPath}");
            throw new \Exception("FTP HELPER: upload failed {$remote_file} > {$xmlPath}");
        }
        else {
            $this->_logger->info("FTP HELPER: Uploaded {$remote_file} > {$xmlPath}");
        }
        return true;
    }

    /**
     *
     * @param string $serverFilename
     * @return bool
     */
    public function deleteFile($serverFilename) {

        $this->_logger->info(__METHOD__);

        ftp_chdir($this->connection,$this->ftpDir);

        return ftp_delete($this->connection, $serverFilename);
    }


    public function close() {
        $closed = ftp_close($this->connection);
        $this->connection = null;
        return $closed;
    }

    /*
     * List, Download and Delete resources from ftp
     */
    public function getResources(){

        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection();
        $this->login($this->username,$this->password);

        $resources = $this->listDir($this->ftpDir);
        $downloadedFiles = array();

        foreach($resources as $resource){
            $basefile = basename($resource);
            $localFile = $this->localDir."/".$basefile;

            // la regex deve avere caratteri delimiter! nel campo backend
            if (preg_match($this->_mask,$basefile)) {

                $downloadedFiles[] = $basefile;

            }

        }

        $this->close();
        return $downloadedFiles;
    }

    public function receiveResource($resource, $copy = null, $ftp_mode = FTP_ASCII, $keep_conn_open = false, $unique_filename_pattern = false) {

        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection();

        $downloadedFile = array();

        if(!is_dir($this->localDir) && trim($this->localDir) != '') {
            mkdir($this->localDir, 0777, true);
        }


        $basefile = basename($resource);
        if ($unique_filename_pattern) {
            $basefile = $this->fixFilename($basefile,$unique_filename_pattern);
        }
        $localFile = $this->localDir."/".$basefile;

        if (file_exists($localFile)) {
            $deleted = $this->deleteFile($resource);
            $this->_logger->info('FTP - file '.$basefile.' already exists');
            return false;
        }

        try {
            if ($this->downloadFile($localFile, $resource, $ftp_mode)) {
                if($this->deleteFile($resource)) {
                    $downloadedFile = $basefile;
                    if($copy){
                        $fileInfo = pathinfo($localFile);
                        $fileCopyName = $copy.'.'.$fileInfo['extension'];
                        $fileCopy = $this->localDir."/".$fileCopyName;
                        if(copy($localFile,$fileCopy)){
                            $downloadedFile = array('basefile'=>$downloadedFile,'copy'=>$fileCopyName);
                        }
                    }
                }
            }
        } catch(\Exception $e) {
            $this->_logger->info($e->getMessage());
        }



        $this->close();


        return $downloadedFile;

    }

    public function isReceived() {

    }

    public function sendResource($resource){

        $this->_logger->info(__METHOD__);

        $this->connection = $this->getConnection($this->host);
        $this->login($this->username,$this->password);

        $downloadedFiles = array();

        if(!is_dir($this->localDir) && trim($this->localDir) != '') {
            mkdir($this->localDir, 0777, true);
        }

        $basefile = basename($resource);
        $localFile = $this->localDir."/".$basefile;

        if (file_exists($localFile)) {
            if ($this->uploadFile($localFile, $resource)) {
                unlink($localFile);
                return true;
            }
        }
        return false;
    }

    public function setFtpDir( $dir ){
        $this->ftpDir = $dir;
        return $this;
    }

    public function setLocalDir( $dir ){
        $this->localDir = $dir;
        return $this;
    }

    // This will change the filename to an unique filename appending the unique pattern before extension
    public function fixFilename($fileName,$unique_filename_pattern = null)
    {
        $fileName = preg_replace('/[^a-z0-9_\\-\\.]+/i', '', $fileName);

        // Add unique identifier to the filename
        if (!$unique_filename_pattern) {
            return $fileName;
        }

        // No extension? then return it with the unique identifier
        $parts = explode('.', $fileName);
        if (count($parts) == 1){
            return $fileName.$unique_filename_pattern;
        }

        // Have extension, then add unique identifier before extension
        $last = array_pop($parts);
        $parts = array(implode('.', $parts), $last);

        $fileName = $parts[0].$unique_filename_pattern.".".$parts[1];

        return $fileName;
    }
}
